@@ -7,7 +7,8 @@ from enum import IntEnum, Enum
 import socket
 from pprint import pformat
 from .buffer import Buffer
-# from .subscribe import Subscribe
+from .math import calculate_spread
+from .Symbol import SYMBOLSLIST
 
 class Field(IntEnum):
     AvgPx = 6
@@ -111,11 +112,8 @@ class OrderType(IntEnum):
     Limit = 2
     Stop = 3
 
-def get_time(add_seconds=None):
-    if add_seconds:
-        return (datetime.utcnow() + timedelta(0, add_seconds)).strftime("%Y%m%d-%H:%M:%S.%f")[:-3]
-    else:
-        return datetime.utcnow().strftime("%Y%m%d-%H:%M:%S.%f")[:-3]
+def get_time():
+        return datetime.utcnow().strftime('%Y%m%d-%H:%M:%S')
 
 class FIX:
 
@@ -123,6 +121,7 @@ class FIX:
 
         def __init__(self, sub: SubID = None, msg_type: str = None, parent = None):
             self.fields = []
+            self.symbol_table = SYMBOLSLIST['default']
             if parent:
                 self.origin = True
                 self.fields.append((Field.BeginString, "FIX.4.4"))
@@ -248,8 +247,12 @@ class FIX:
             if len(data) == 0:
                 logging.error("Disconnected")
                 break
-            self.qstream.write(data)
-            self.parse_quote_message()
+            try:
+                self.qstream.write(data)
+                self.parse_quote_message()
+            except:
+                print("Market is Close or Disconnected")
+                break
 
     def tworker(self):
         while True:
@@ -260,8 +263,12 @@ class FIX:
             if len(data) == 0:
                 logging.error("Disconnected")
                 break
-            self.tstream.write(data)
-            self.parse_trade_message()
+            try:
+                self.tstream.write(data)
+                self.parse_trade_message()
+            except:
+                print("Market is Close or Disconnected")
+                break
 
     def parse_quote_message(self):
         while len(self.qstream) > 0:
@@ -377,7 +384,18 @@ class FIX:
         if not msg[Field.MDEntryID] and msg[Field.NoMDEntries] != "0":
             self.spot_price_list[name] = {}
             for e in entries:
-                self.spot_price_list[name]["bid" if e[Field.MDEntryType] == "0" else "ask"] = float(e[Field.MDEntryPx])
+                ask_idx = 1 if e[0][Field.MDEntryType] == '0' else 0
+                bid_idx = (ask_idx + 1) % 2
+                spread = calculate_spread(
+                e[bid_idx][Field.MDEntryPx],
+                e[ask_idx][Field.MDEntryPx],
+                self.symbol_table[int(msg.get_field(Field.Symbol))]['pip_position']
+                 )
+                self.spot_price_list[name]["time"] = int(round(time.time() * 1000))
+                self.spot_price_list[name]["spread"] = spread
+                self.spot_price_list[name]["bid"] = e[bid_idx][Field.MDEntryPx]
+                self.spot_price_list[name]["ask"] = e[ask_idx][Field.MDEntryPx]
+                
             self.position_list_callback(self.position_list, self.spot_price_list, self.client_id)
             self.order_list_callback(self.order_list, self.spot_price_list, self.client_id)
             return
